function [Emu,ESIGMA,n]=EMdis_M(u,ntime,a,b,c)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% This function is used to estimate the mean and covariance %%%
%%% matrix of latent traits for longituinal IRT model (Stage  %%% 
%%% 1) using EM algorithm.                                    %%%
%%% The size of a(b,c) is time*item, and the values are known.%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[person,item,time]=size(u);
domain=size(a,1);
J=item/domain;
%%%             FUNCTION HANDLE FOR IRT MODELS.              %%%%
d=1.7;
P=@(th,a,b,c) ones(size(th,1),1)*c+(1-ones(size(th,1),1)*c)./(1+exp(-d*th*a+d*ones(size(th,1),1)*(a.*b)));  %person*item
itemlike=@(u,th,a,b,c) P(th,a,b,c).^u.*(1-P(th,a,b,c)).^(1-u);  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% If the measurement model is 2PL model, 'c' parameter    %%%
%%% can not be included in input arguments, then set 'c'    %%%
%%% parameter to be 0.                                      %%%
if nargin<5  
    c=zeros(time,item);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Initial value of the mean and covariance matrix.        %%%
T=[eye(domain) 0*eye(domain);eye(domain) eye(domain);eye(domain) 2*eye(domain);eye(domain) 3*eye(domain)];
new_mu=zeros(1,time*domain);
new_sigma=eye(time*domain);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
indic=1; n=1;
SamNo=1000;  % The number of quadrature points in Monte Carlo method.
while indic==1 && n<ntime
    N_mu=zeros(person,time*domain); par_mu=new_mu;
    D_mu=zeros(person,1);    
    for m=1:SamNo
        sam_theta=mvnrnd(ones(person,1)*new_mu,new_sigma);   %person*(time*domain)
        post=ones(person,1);
        for t=1:time
            for d=1:domain
                at=a(d,(d-1)*J+1:d*J,t);
                u0=u(:,(d-1)*J+1:d*J,t);
                post=post.*prod(itemlike(u0,sam_theta(:,(t-1)*domain+d),at,b(t,(d-1)*J+1:d*J),c(t,(d-1)*J+1:d*J)),2);  %person*1
            end
        end
        weight=post;
        D_mu=D_mu+weight;
        N_mu=N_mu+sam_theta.*(weight*ones(1,time*domain));
    end
    new_mu=sum(N_mu./(D_mu*ones(1,time*domain)))/person;    
    clear sam_theta post weight u0
    
     par_sigma=new_sigma;Spart=zeros(time*domain,time*domain);
    for i=1:person
        N_sigma=zeros(time*domain,time*domain); D_sigma=0;
        for m=1:SamNo
            post=1;
            sam_theta=mvnrnd(new_mu,new_sigma);   %1*(time*domain)
            for t=1:time
                for d=1:domain
                    u0(1,:)=u(i,(d-1)*J+1:d*J,t);
                    at=a(d,(d-1)*J+1:d*J,t);
                    post=post*prod(itemlike(u0,sam_theta((t-1)*domain+d),at,b(t,(d-1)*J+1:d*J),c(t,(d-1)*J+1:d*J)),2);  %1*1
                end
            end
            weight=post;
            D_sigma=D_sigma+weight;
            N_sigma=N_sigma+(sam_theta-new_mu)'*(sam_theta-new_mu)*weight;  %(time*domain)*(time*domain)            
        end        
        Spart=Spart+N_sigma/D_sigma;
    end
    clear sam_theta post weight
    new_sigma=Spart/person;

    
    if max(abs(new_mu-par_mu))+max(abs(new_sigma(1:end)-par_sigma(1:end)))<0.01
        indic=0;
    else n=n+1;
    end
    
end
Emu=new_mu;
ESIGMA=new_sigma;
    
    
    