

function New_Ability_Estimation_Value=MAP_theta(Ability_Estimation_Value,U,V,Test_Length,A,B,C,D,Accuracy_Of_Iteration,gtheta,vtheta)

% This function is used to estimate the examinee ability by using MAP method combined with bisection method
% gtheta=mean of the prior
% vtheta is the prior variance
% Ability_Estimation_Value is the current ability estimate of examinee
% Array U stores the IDs of items which the current examinee has answered
% Array V records the scores of items which the current examinee has
% answered, i.e., Responses
% Test_Length is current test length
% Array A stores the item discrimination parameters of all items(this is 3PLM case)
% Array B stores the item difficulty parameters of all items
% Array C stores the item guessing parameters of all items
% D is the scale constant. When D=1, the scale is logistic scale; when D=1.702, the scale is norm scale
% Accuracy_of_Iteration represents the accuracy of iteration(equals to 0.01 or 0.005)
% New_Ability_Estimation_Value is the new examinee ability estimation
%D=1.702;
%D=1;
%C=zeros(Test_Length,1);

New_Ability_Estimation_Value=Ability_Estimation_Value;
if (abs(New_Ability_Estimation_Value)~=3)
    Try=New_Ability_Estimation_Value;
else
    Try=0;
    New_Ability_Estimation_Value=0;
end

% compute the value of g(Try)
Dtry=0;
for j=1:Test_Length
    Prob=ICF_Computation(Try,A(U(j)),B(U(j)),C(U(j)),D);
    Dtry=Dtry+D*A(U(j))*(Prob-C(U(j)))*(V(j)-Prob)/((1-C(U(j)))*Prob);
end

Dtry=Dtry-(New_Ability_Estimation_Value-gtheta)/vtheta;

% Find the interval bounded by Tlow and Thi that contains zero crossing
if (Dtry>0)                                % find from left to right
    while (true)
        Tlow=Try;
        if (Tlow<3)
            Try=Tlow+0.5;
            Dtry=0;
            for j=1:Test_Length
                Prob=ICF_Computation(Try,A(U(j)),B(U(j)),C(U(j)),D);
                Dtry=Dtry+D*A(U(j))*(Prob-C(U(j)))*(V(j)-Prob)/((1-C(U(j)))*Prob);
            end
            
            Dtry=Dtry-(Try-gtheta)/vtheta;
            if (Dtry<0) 
                break;
            end
        else
            New_Ability_Estimation_Value=3;
            break;
        end
    end
    Thi=Try;
else                                       % find from right to left
    while (true)
        Thi=Try;
        if (Thi>-3)
            Try=Thi-0.5;
            Dtry=0;
            for j=1:Test_Length
                Prob=ICF_Computation(Try,A(U(j)),B(U(j)),C(U(j)),D);
                Dtry=Dtry+D*A(U(j))*(Prob-C(U(j)))*(V(j)-Prob)/((1-C(U(j)))*Prob);
            end
            Dtry=Dtry-(Try-gtheta)/vtheta;
            if (Dtry>0)
                break;
            end
        else
            New_Ability_Estimation_Value=-3;
            break;
        end
    end
    Tlow=Try;
end

if (abs(New_Ability_Estimation_Value)~=3)
    Try=(Thi+Tlow)/2;                   % bisection
    Dtry=0;
    for j=1:Test_Length
    	Prob=ICF_Computation(Try,A(U(j)),B(U(j)),C(U(j)),D);
        Dtry=Dtry+D*A(U(j))*(Prob-C(U(j)))*(V(j)-Prob)/((1-C(U(j)))*Prob);
    end
    Dtry=Dtry-(Try-gtheta)/vtheta;
    while (true)
        if (abs(Dtry)<Accuracy_Of_Iteration)
            break;
        end
        D2try=0;
        Dtry=0;
        for j=1:Test_Length
            Prob=ICF_Computation(Try,A(U(j)),B(U(j)),C(U(j)),D);
            Dtry=Dtry+D*A(U(j))*(Prob-C(U(j)))*(V(j)-Prob)/((1-C(U(j)))*Prob);        % Dtry is the first derivative  
            D2try=D2try+D^2*(A(U(j)))^2*(1-Prob)*(Prob-C(U(j)))^2/((1-C(U(j)))^2*Prob); % % Dtry is the second derivative
        end 
        Dtry=Dtry-(Try-gtheta)/vtheta;
        D2try=D2try-1/vtheta;
        if (Dtry>0)
            Tlow=Try;
        else
            Thi=Try;
        end
        % Next is the Newton-Rapson iteration
        if (abs(Dtry)<=1)
            Try2=Try-Dtry/D2try;
            if ((Try2>Tlow) && (Try2<Thi))
                Try=Try2;
            else
                Try=(Tlow+Thi)/2;
            end
        else
            Try=(Tlow+Thi)/2;
        end
    end
    New_Ability_Estimation_Value=Try;
end